"""Numeric Values of Onyx devices."""
from typing import Optional

from onyx_client.data.animation_value import AnimationValue


class NumericValue:
    """The representation of a numeric value."""

    def __init__(
        self,
        value: int,
        minimum: int,
        maximum: int,
        read_only: bool,
        animation: Optional[AnimationValue] = None,
    ):
        """Initialize the numeric value."""
        self.value = value
        self.minimum = minimum
        self.maximum = maximum
        self.read_only = read_only
        self.animation = animation

    @staticmethod
    def create(properties: dict):
        """Create a numeric value from properties."""
        if properties is None:
            return None

        return NumericValue(
            properties.get("value", None),
            properties.get("minimum", 0),
            properties.get("maximum", 100),
            properties.get("read_only", False),
            AnimationValue.create(properties.get("animation", None)),
        )

    def update_with(self, other: Optional):
        """Updates this value with the target."""
        if other is not None:
            self.value = self.value if other.value is None else other.value
            self.minimum = self.minimum if other.minimum is None else other.minimum
            self.maximum = self.maximum if other.maximum is None else other.maximum
            self.read_only = (
                self.read_only if other.read_only is None else other.read_only
            )
            self.animation = (
                self.animation if other.animation is None else other.animation
            )

    def __eq__(self, other):
        if isinstance(self, other.__class__):
            return (
                self.value == other.value
                and self.minimum == other.minimum
                and self.maximum == other.maximum
                and self.read_only == other.read_only
                and self.animation == other.animation
            )
        return False



"""Onyx Client helpers."""
from typing import Optional

import aiohttp

from onyx_client import OnyxClient
from onyx_client.configuration.configuration import Configuration


async def exchange_code(
    code: str, client_session: aiohttp.ClientSession = None
) -> Optional[Configuration]:
    """Exchange an API code for an access token and fingerprint."""
    session = client_session if client_session is not None else aiohttp.ClientSession()
    config = await OnyxClient.authorize(code, session)
    if await OnyxClient(config, session).verify():
        return config
    return None



"""Click class."""
from onyx_client.data.device_mode import DeviceMode
from onyx_client.device.device import Device
from onyx_client.enum.device_type import DeviceType


class Click(Device):
    """A ONYX controlled click device."""

    def __init__(
        self, identifier: str, name: str, device_type: DeviceType, offline: bool
    ):
        """Initialize the click device."""
        super(Click, self).__init__(
            identifier,
            name,
            device_type,
            DeviceMode(device_type),
            list(),
        )
        self.offline = offline

    def update_with(self, update):
        super().update_with(update)

        self.offline = self.offline if update.offline is None else update.offline

    @staticmethod
    def keys() -> list:
        return [
            "offline",
        ]



"""HTTP Client for Hella's ONYX.CENTER API."""
import aiohttp

from onyx_client.client import OnyxClient
from onyx_client.configuration.configuration import Configuration


def create_client(
    config: Configuration = None,
    fingerprint: str = None,
    access_token: str = None,
    client_session: aiohttp.ClientSession = None,
) -> OnyxClient:
    if config is None:
        config = Configuration(fingerprint, access_token)
    session = client_session if client_session is not None else aiohttp.ClientSession()
    return OnyxClient(config, session)



"""Onyx Client API class."""
import json
import logging
from typing import Any, Optional

import aiohttp

from onyx_client.configuration.configuration import Configuration
from onyx_client.data.boolean_value import BooleanValue
from onyx_client.data.date_information import DateInformation
from onyx_client.data.device_command import DeviceCommand
from onyx_client.data.device_mode import DeviceMode
from onyx_client.data.numeric_value import NumericValue
from onyx_client.data.supported_versions import SupportedVersions
from onyx_client.device.click import Click
from onyx_client.device.device import Device
from onyx_client.device.light import Light
from onyx_client.device.shutter import Shutter
from onyx_client.device.weather import Weather
from onyx_client.enum.action import Action
from onyx_client.enum.device_type import DeviceType
from onyx_client.group.group import Group
from onyx_client.utils.const import API_HEADERS, API_URL, API_VERSION
from onyx_client.utils.filter import present

_LOGGER = logging.getLogger(__name__)


class OnyxClient:
    """The ONYX.CENTER API Client.

    After initializing, call ::verify to check if:
      - the provided connection parameters are correct
      - the ONYX.CENTER supports the client's API version"""

    def __init__(self, config: Configuration, client_session: aiohttp.ClientSession):
        """Initialize the API client."""
        self.config = config
        self.client_session = client_session

    @staticmethod
    async def authorize(
        code: str, client_session: aiohttp.ClientSession
    ) -> Optional[Configuration]:
        """Authorize the client using an API code."""
        async with client_session.post(
            f"{API_URL}/authorize",
            data={"code": code},
            headers=API_HEADERS,
        ) as response:
            if not OnyxClient._check_response(response):
                _LOGGER.error("Could not authorize client for ONYX API.")
                return None
            data = await response.json()
            return Configuration(data.get("fingerprint", None), data.get("token"))

    @property
    def _headers(self) -> dict:
        """Get all common headers."""
        return {"Authorization": f"Bearer {self.config.access_token}", **API_HEADERS}

    def _base_url(self, with_api: bool = True) -> str:
        """Get the API base URL for this ONYX.CENTER."""
        api = f"{API_URL}/box/{self.config.fingerprint}/api"
        if with_api:
            api = f"{api}/{API_VERSION}"
        return api

    def _url(self, path: str = "", with_api: bool = True) -> str:
        """Get the request URL."""
        return f"{self._base_url(with_api=with_api)}{path}"

    @staticmethod
    def _check_response(response: aiohttp.ClientResponse) -> bool:
        """Check the response for a success HTTP status code.

        Success codes are:
          - 200"""
        if response.status == 200:
            _LOGGER.debug("Received HTTP response from ONYX API: %s", response.status)
            return True
        else:
            _LOGGER.error(
                "Received erroneous HTTP response from ONYX API: %s", response.status
            )
            return False

    @staticmethod
    def _numeric_value(key: str, properties: dict = None):
        return (
            NumericValue.create(properties.get(key, None))
            if properties is not None
            else None
        )

    @staticmethod
    def _boolean_value(key: str, properties: dict = None):
        return (
            BooleanValue.create(properties.get(key, None))
            if properties is not None
            else None
        )

    @staticmethod
    def _init_device(
        identifier: str,
        name: str = None,
        device_type: DeviceType = None,
        properties: dict = None,
        actions: list = None,
        data: dict = None,
    ) -> Device:
        """Initialize the device correctly."""
        device_mode_value = (
            DeviceType.convert(properties.get("device_type", dict()).get("type", None))
            if properties is not None
            else device_type
        )
        device_mode_values = (
            [
                DeviceType.convert(value)
                for value in properties.get("device_type", dict()).get("values", list())
            ]
            if properties is not None
            else None
        )
        device_mode = DeviceMode(device_mode_value, device_mode_values)
        if OnyxClient._is_shutter(device_type, properties):
            return Shutter(
                identifier,
                name,
                device_type,
                device_mode,
                actions,
                OnyxClient._numeric_value("target_position", properties),
                OnyxClient._numeric_value("target_angle", properties),
                OnyxClient._numeric_value("actual_angle", properties),
                OnyxClient._numeric_value("actual_position", properties),
            )
        elif OnyxClient._is_weather(device_type, properties):
            return Weather(
                identifier,
                name,
                device_type,
                device_mode,
                actions,
                OnyxClient._numeric_value("wind_peak", properties),
                OnyxClient._numeric_value("sun_brightness_peak", properties),
                OnyxClient._numeric_value("sun_brightness_sink", properties),
                OnyxClient._numeric_value("air_pressure", properties),
                OnyxClient._numeric_value("humidity", properties),
                OnyxClient._numeric_value("temperature", properties),
            )
        elif OnyxClient._is_light(device_type, properties):
            return Light(
                identifier,
                name,
                device_type,
                device_mode,
                actions,
                OnyxClient._numeric_value("target_brightness", properties),
                OnyxClient._numeric_value("actual_brightness", properties),
                OnyxClient._numeric_value("dim_duration", properties),
            )
        elif OnyxClient._is_click(device_type, data):
            offline = data.get("offline", True) if data is not None else True
            return Click(identifier, name, device_type, offline)
        else:
            return Device(identifier, name, DeviceType.UNKNOWN, device_mode, actions)

    @staticmethod
    def _is_shutter(device_type: DeviceType, properties: dict) -> bool:
        if device_type is not None:
            return device_type.is_shutter()
        if properties is not None:
            for key in properties.keys():
                if key in Shutter.keys():
                    return True
        return False

    @staticmethod
    def _is_light(device_type: DeviceType, properties: dict) -> bool:
        if device_type is not None:
            return device_type == DeviceType.BASIC_LIGHT
        if properties is not None:
            for key in properties.keys():
                if key in Light.keys():
                    return True
        return False

    @staticmethod
    def _is_weather(device_type: DeviceType, properties: dict) -> bool:
        if device_type is not None:
            return device_type == DeviceType.WEATHER
        if properties is not None:
            for key in properties.keys():
                if key in Weather.keys():
                    return True
        return False

    @staticmethod
    def _is_click(device_type: DeviceType, data: dict) -> bool:
        if device_type is not None:
            return device_type == DeviceType.CLICK
        if data is not None:
            for key in data.keys():
                if key in Click.keys():
                    return True
        return False

    async def _perform_get_request(
        self, path: str, with_api: bool = True
    ) -> Optional[Any]:
        """Perform a GET request."""
        async with self.client_session.get(
            self._url(path, with_api=with_api), headers=self._headers
        ) as response:
            if not self._check_response(response):
                return None
            return await response.json()

    async def _perform_delete_request(self, path: str) -> Optional[Any]:
        """Perform a DELETE request."""
        async with self.client_session.delete(
            self._url(path), headers=self._headers
        ) as response:
            if not self._check_response(response):
                return None
            return await response.json()

    async def _perform_post_request(self, path: str, data: dict) -> Optional[Any]:
        """Perform a POST request."""
        async with self.client_session.post(
            self._url(path), json=data, headers=self._headers
        ) as response:
            if not self._check_response(response):
                return None
            return await response.json()

    async def supported_versions(self) -> Optional[SupportedVersions]:
        """Get all supported versions by the ONYX.CENTER."""
        data = await self._perform_get_request("/versions", with_api=False)
        if data is None:
            _LOGGER.error(
                "Could not call ONYX API for device %s: /versions.",
                self.config.fingerprint,
            )
            return None

        return SupportedVersions(data.get("versions", list()))

    async def verify(self) -> bool:
        """Check if the ONYX.CENTER supports the version
        and the connection parameters are working."""
        versions = await self.supported_versions()
        return versions.supports(API_VERSION) if versions is not None else False

    async def date_information(self) -> Optional[DateInformation]:
        """Get all date related information of the ONYX.CENTER."""
        data = await self._perform_get_request("/clock")
        if data is None:
            _LOGGER.error(
                "Could not call ONYX API for device %s: /clock.",
                self.config.fingerprint,
            )
            return None

        return DateInformation(
            float(data.get("time", "0")),
            data.get("zone", None),
            int(data.get("zone_offset", "0")),
        )

    async def devices(self, include_details: bool = False) -> Optional[list]:
        """Get all devices controlled by the ONYX.CENTER."""
        data = await self._perform_get_request("/devices")
        if data is None:
            _LOGGER.error(
                "Could not call ONYX API for device %s: /devices.",
                self.config.fingerprint,
            )
            return None

        if include_details:
            return [
                device
                for device in [await self.device(key) for key, _ in data.items()]
                if present(device)
            ]
        else:
            return [
                self._init_device(
                    key,
                    value.get("name", None),
                    DeviceType.convert(value.get("type", None)),
                )
                for key, value in data.items()
            ]

    async def device(self, identifier: str) -> Optional[Device]:
        """Get the device properties for a provided ID."""
        data = await self._perform_get_request(f"/devices/{identifier}")
        if data is None:
            _LOGGER.error(
                "Could not call ONYX API for device %s: /devices/%s.",
                self.config.fingerprint,
                identifier,
            )
            return None

        actions = [Action.convert(action) for action in data.get("actions", list())]
        return self._init_device(
            identifier,
            data.get("name", None),
            DeviceType.convert(data.get("type", None)),
            data.get("properties", None),
            actions,
            data,
        )

    async def send_command(self, identifier: str, command: DeviceCommand) -> bool:
        """Send a command to the device with the provided ID."""
        data = await self._perform_post_request(
            f"/devices/{identifier}/command", command.data()
        )
        if data is None:
            _LOGGER.error(
                "Could not call ONYX API for device %s: /devices/%s/command.",
                self.config.fingerprint,
                identifier,
            )
        return data is not None

    async def cancel_command(self, identifier: str) -> bool:
        """Cancel a command to the device with the provided ID."""
        data = await self._perform_delete_request(f"/devices/{identifier}/command")
        if data is None:
            _LOGGER.error(
                "Could not call ONYX API for device %s: /devices/%s/command.",
                self.config.fingerprint,
                identifier,
            )
        return data is not None

    async def groups(self) -> Optional[list]:
        """Get all groups controlled by the ONYX.CENTER."""
        data = await self._perform_get_request("/groups")
        if data is None:
            _LOGGER.error(
                "Could not call ONYX API for device %s: /groups.",
                self.config.fingerprint,
            )
            return None

        return [
            Group(key, value.get("name", None), value.get("devices", list()))
            for key, value in data.items()
        ]

    async def group(self, identifier: str) -> Optional[Group]:
        """Get the group properties for a provided ID."""
        data = await self._perform_get_request(f"/groups/{identifier}")
        if data is None:
            _LOGGER.error(
                "Could not call ONYX API for device %s: /groups/%s.",
                self.config.fingerprint,
                identifier,
            )
            return None

        return Group(identifier, data.get("name", None), data.get("devices", list()))

    async def send_group_command(self, identifier: str, command: DeviceCommand) -> bool:
        """Send a command to the group with the provided ID."""
        data = await self._perform_post_request(
            f"/groups/{identifier}/command", command.data()
        )
        if data is None:
            _LOGGER.error(
                "Could not call ONYX API for device %s: /groups/%s/command.",
                self.config.fingerprint,
                identifier,
            )
            return False

        unsuccessful = [
            key
            for (key, value) in data.get("results", dict()).items()
            if value.get("status_code", 501) != 200
        ]
        if len(unsuccessful) > 0:
            _LOGGER.error(
                "Could not execute command for all devices in group %s: %s",
                identifier,
                unsuccessful,
            )
        return len(unsuccessful) == 0

    async def cancel_group_command(self, identifier: str) -> bool:
        """Cancel a command to the group with the provided ID."""
        data = await self._perform_delete_request(f"/groups/{identifier}/command")
        if data is None:
            _LOGGER.error(
                "Could not call ONYX API for device %s: /groups/%s/command.",
                self.config.fingerprint,
                identifier,
            )
        return data is not None

    async def events(self, include_details: bool = False) -> Device:
        """Stream events continuously."""
        async with self.client_session.get(
            self._url("/events"), headers=self._headers
        ) as response:
            if not self._check_response(response):
                yield None
                return
            async for message in response.content:
                cleaned_message = str(message.strip(), "UTF-8").strip()
                if len(cleaned_message) > 0 and cleaned_message.startswith("data:"):
                    cleaned_message = cleaned_message[len("data:") :].strip()
                    events = json.loads(cleaned_message)
                    for key, value in events.get("devices", dict()).items():
                        try:
                            if value is not None:
                                device = (
                                    await self.device(key)
                                    if include_details
                                    else self._init_device(
                                        key,
                                        value.get("name", None),
                                        DeviceType.convert(value.get("type", None)),
                                        value.get("properties", None),
                                        value,
                                    )
                                )
                                yield device
                        except AttributeError:
                            _LOGGER.error(
                                "Received unknown device data. Dropping device %s", key
                            )



"""Animation Values of Onyx devices."""
from onyx_client.data.animation_keyframe import AnimationKeyframe


class AnimationValue:
    """The representation of an animation value."""

    def __init__(self, start: float, current_value: int, keyframes: list):
        """Initialize the animation value."""
        self.start = start
        self.current_value = current_value
        self.keyframes = keyframes

    @staticmethod
    def create(properties: dict):
        """Create an animation value from properties."""
        if properties is None:
            return None

        return AnimationValue(
            properties.get("start", None),
            properties.get("current_value"),
            [
                AnimationKeyframe.create(keyframe)
                for keyframe in properties.get("keyframes", list())
            ],
        )

    def __eq__(self, other):
        if isinstance(self, other.__class__):
            return (
                self.start == other.start and self.current_value == other.current_value
            )
        return False



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['onyx_client',
 'onyx_client.configuration',
 'onyx_client.data',
 'onyx_client.device',
 'onyx_client.enum',
 'onyx_client.exception',
 'onyx_client.group',
 'onyx_client.helpers',
 'onyx_client.utils']

package_data = \
{'': ['*']}

install_requires = \
['aiohttp>=3.7.3,<4.0.0', 'brotlipy>=0.7.0,<0.8.0', 'cchardet>=2.1.7,<3.0.0']

setup_kwargs = {
    'name': 'onyx-client',
    'version': '3.0.4',
    'description': "HTTP Client for Hella's ONYX.CENTER API.",
    'long_description': '# Onyx Client\n\n[![](https://img.shields.io/github/license/muhlba91/onyx-client?style=for-the-badge)](LICENSE)\n[![](https://img.shields.io/github/workflow/status/muhlba91/onyx-client/Python%20package?style=for-the-badge)](https://github.com/muhlba91/onyx-client/actions)\n[![](https://img.shields.io/coveralls/github/muhlba91/onyx-client?style=for-the-badge)](https://github.com/muhlba91/onyx-client/)\n[![](https://img.shields.io/pypi/pyversions/onyx-client?style=for-the-badge)](https://pypi.org/project/onyx-client/)\n[![](https://img.shields.io/pypi/v/onyx-client?style=for-the-badge)](https://pypi.org/project/onyx-client/)\n[![](https://img.shields.io/github/release-date/muhlba91/onyx-client?style=for-the-badge)](https://github.com/muhlba91/onyx-client/releases)\n[![](https://img.shields.io/pypi/dm/onyx-client?style=for-the-badge)](https://pypi.org/project/onyx-client/)\n<a href="https://www.buymeacoffee.com/muhlba91" target="_blank"><img src="https://cdn.buymeacoffee.com/buttons/default-orange.png" alt="Buy Me A Coffee" height="28" width="150"></a>\n\nThis repository contains a **Python HTTP client** for [Hella](https://www.hella.info)\'\ns [ONYX.CENTER API](https://github.com/hella-info/onyx_api).\n\n---\n\n## Installation\n\nThe package is published in **(Test)PyPi** and can be installed via:\n\n```bash\npip install onyx-client\n```\n\n## Configuration\n\nThe configuration defines **connection properties** as a `dict` for the application running.\n\n**Attention**: make sure to **read**\nthe [Onyx API Access Control](https://github.com/hella-info/onyx_api#access-control) description to **retrieve the\nfingerprint and access token**!\n\n| Option | Description |\n|--------|-------------|\n| fingerprint | The fingerprint of the ONYX.CENTER. |\n| access_token | The permanent access token. |\n/ client_session / The initialized `aiohttp.ClientSession`. (Default: `None`, create new session.) |\n\n### Access Control Helper\n\nThe method `onyx_client.helpers.exchange_code` takes the **API code** and performs the exchange to a **fingerprint and\naccess token**. Please follow the **aforementioned documentation** to retrieve the code.\n\n## Usage\n\nYou can **instantiate** the client using the `onyx_client.create_client` method like:\n\n```python\nimport aiohttp\nfrom onyx_client import create_client\nfrom onyx_client.helpers import exchange_code\n\n# by providing the fingerprint and access token only\nclient = create_client(fingerprint="fingerprint", access_token="access_token")\n\n# by providing the fingerprint, access token and aiohttp client session\nclient = create_client(fingerprint="fingerprint", access_token="access_token", client_session=aiohttp.ClientSession())\n\n# by providing the configuration object\nclient_session = aiohttp.ClientSession()\n# e.g. by exchanging the code first\nconfig = exchange_code("code", client_session)\nclient = create_client(config=config, client_session=client_session) if client_session is not None else None\n```\n\nAn **example** is shown in the **`examples` directory**.\n\n---\n\n## Development\n\nThe project uses [poetry](https://poetry.eustace.io/) and to install all dependencies and the build environment, run:\n\n```bash\n$ pip install poetry\n$ poetry install\n```\n\n### Testing\n\n1) Install all dependencies as shown above.\n2) Run `pytest` by:\n\n```bash\n$ poetry run pytest\n# or\n$ pytest\n```\n\n### Linting and Code Style\n\nThe project uses [flakehell](https://github.com/life4/flakehell) as a wrapper for flake8,\nand [black](https://github.com/psf/black) for automated code style fixing, also\nusing [pre-commit](https://pre-commit.com/).\n\n1) Install all dependencies as shown above.\n2) (Optional) Install pre-commit hooks:\n\n```bash\n$ poetry run pre-commit install\n```\n\n3) Run black:\n\n```bash\n$ poetry run black .\n```\n\n4) Run flakehell:\n\n```bash\n$ poetry run flakehell lint\n```\n\n### Building\n\nThis package uses [poetry-dynamic-versioning](https://github.com/mtkennerly/poetry-dynamic-versioning) which infers the\nversion number based on the Git tags. Hence, to have a proper versioning for the distribution, use Python\'s build system\nlike:\n\n```bash\n$ pip install build\n$ python -m build\n```\n\nYour distribution will be in the `dist` directory.\n\n### Commit Message\n\nThis project follows [Conventional Commits](https://www.conventionalcommits.org/), and your commit message must also\nadhere to the additional rules outlined in `.conform.yaml`.\n\n---\n\n## Release\n\nTo draft a release, use [standard-version](https://github.com/conventional-changelog/standard-version):\n\n```bash\n$ standard-version\n# alternatively\n$ npx standard-version\n```\n\nFinally, push with tags:\n\n```bash\n$ git push --follow-tags\n```\n\n---\n\n## Contributions\n\nPlease feel free to contribute, be it with Issues or Pull Requests! Please read\nthe [Contribution guidelines](CONTRIBUTING.md)\n\n## Supporting\n\nIf you enjoy the application and want to support my efforts, please feel free to buy me a coffe. :)\n\n<a href="https://www.buymeacoffee.com/muhlba91" target="_blank"><img src="https://cdn.buymeacoffee.com/buttons/default-orange.png" alt="Buy Me A Coffee" height="75" width="300"></a>\n',
    'author': 'Daniel Muehlbachler-Pietrzykowski',
    'author_email': 'daniel.muehlbachler@niftyside.io',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/muhlba91/onyx-client',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



"""Light class."""
from onyx_client.data.device_mode import DeviceMode
from onyx_client.data.numeric_value import NumericValue
from onyx_client.device.device import Device
from onyx_client.enum.device_type import DeviceType


class Light(Device):
    """A ONYX controlled light."""

    def __init__(
        self,
        identifier: str,
        name: str,
        device_type: DeviceType,
        device_mode: DeviceMode,
        actions: list,
        target_brightness: NumericValue = None,
        actual_brightness: NumericValue = None,
        dim_duration: NumericValue = None,
    ):
        """Initialize the light device."""
        super(Light, self).__init__(identifier, name, device_type, device_mode, actions)
        self.target_brightness = target_brightness
        self.actual_brightness = actual_brightness
        self.dim_duration = dim_duration

    def update_with(self, update):
        super().update_with(update)

        self.target_brightness = (
            self.target_brightness
            if update.target_brightness is None
            else update.target_brightness
        )
        self.actual_brightness = (
            self.actual_brightness
            if update.actual_brightness is None
            else update.actual_brightness
        )
        self.dim_duration = (
            self.dim_duration if update.dim_duration is None else update.dim_duration
        )

    @staticmethod
    def keys() -> list:
        return [
            "target_brightness",
            "actual_brightness",
            "dim_duration",
        ]
